package org.agilefaqs.rft;

import static java.util.Arrays.asList;
import static org.agilefaqs.rft.internal.MatchingCriterion.MATCHING_DOMAIN;
import static org.agilefaqs.rft.internal.MatchingCriterion.MATCHING_IP;
import static org.agilefaqs.rft.internal.MatchingCriterion.MATCHING_SECOND_LEVEL_DOMAIN;

import java.util.List;

import org.agilefaqs.rft.internal.Domain;
import org.agilefaqs.rft.internal.IPAddress;
import org.agilefaqs.rft.internal.MatchingCriterion;

public class IPAddressExtractor {
    private IPAddress senderIP = IPAddress.NULL;
    private int distance;

    public IPAddressExtractor(final List<EmailHeader> receivedHeaders, final Domain recepientDomain) {
        Domain recepientMXRecord = recepientDomain.retrieveFirstMXRecord();
        for (MatchingCriterion critierion : asList(MATCHING_DOMAIN, MATCHING_IP, MATCHING_SECOND_LEVEL_DOMAIN)) {
            Match result = foreach(receivedHeaders).and(recepientMXRecord).match(critierion);
            if (result.success()) {
                storeSenderIPWithDistance(result);
                break;
            }
        }
    }

    private void storeSenderIPWithDistance(final Match matchingResult) {
        IPAddress ip = matchingResult.domain.resolveIP();
        if (ip.isValid()) {
            senderIP = ip;
        } else {
            senderIP = matchingResult.ip;
        }
        distance = matchingResult.distance;
    }

    private CriterionMatcher foreach(final List<EmailHeader> mailHeaders) {
        return new CriterionMatcher(mailHeaders);
    }

    public IPAddress getSenderIP() {
        return senderIP;
    }

    public int getDistance() {
        return distance;
    }
}
