package org.agilefaqs.rft.internal;

import java.net.UnknownHostException;
import java.util.regex.Pattern;

import org.agilefaqs.rft.Network;

public final class DomainFactory {
    private static final String DOMAIN_NAME_REGEX = "[\\w.-]+\\.[A-Za-z]{2,6}";
    private static final int MAX_DOMAIN_NAME_LENGTH = 255;

    private DomainFactory() {
    }

    public static Domain build(final String domainName, final Network network) {
        if (isValidDomain(domainName)) {
            return new Domain(domainName, network);
        }
        IPAddress ip = IPAddress.parse(domainName);
        if (ip.isValid()) {
            return retrieveDomainName(ip, network);
        }
        return Domain.NULL;
    }

    private static Domain retrieveDomainName(final IPAddress ip, final Network network) {
        try {
            String hostName = network.hostNameFor(ip.toString());
            if (ip.toString().equals(hostName)) {
                return Domain.NULL;
            }
            return new Domain(hostName, network);
        } catch (UnknownHostException e) {
            return Domain.NULL;
        }
    }

    private static boolean isValidDomainName(final String domain) {
        return !(domain.length() == 0 || domain.length() > MAX_DOMAIN_NAME_LENGTH || !Pattern.matches(DOMAIN_NAME_REGEX, domain));
    }

    private static boolean isValidDomain(final String domain) {
        if (domain == null || domain.trim().length() == 0) {
            return false;
        }
        return isValidDomainName(domain);
    }
}
