package org.agilefaqs.rft.internal;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class IPAddress {
    private static final String IP_ADDRESS_REGEX = "\\b(?:\\d{1,3}\\.){3}\\d{1,3}\\b";
    private static final Pattern VALID_IP = Pattern.compile(IP_ADDRESS_REGEX);
    private static final String LOCAL_HOST = "127.0.0.1";
    public static final IPAddress NULL = new IPAddress("");

    private final String ip;

    private IPAddress(final String address) {
        ip = address;
    }

    public static IPAddress parse(final String address) {
        if (address == null) {
            return NULL;
        }
        Matcher matcher = VALID_IP.matcher(address);

        if (!matcher.find()) {
            return NULL;
        }
        return new IPAddress(matcher.group(0));
    }

    @Override
    public String toString() {
        return ip;
    }

    public boolean isLocalhost() {
        return LOCAL_HOST.equals(ip);
    }

    public boolean isValid() {
        return this != NULL;
    }

    @Override
    public boolean equals(final Object other) {
        if (this == other) {
            return true;
        }
        if (null == other) {
            return false;
        }
        if (!this.getClass().equals(other.getClass())) {
            return false;
        }
        return equals((IPAddress) other);
    }

    private boolean equals(final IPAddress other) {
        return ip.equals(other.ip);
    }

    @Override
    public int hashCode() {
        return ip.hashCode();
    }
}
