package org.agilefaqs.rft.internal;

import org.agilefaqs.rft.EmailHeader;

public abstract class MatchingCriterion {
    public static final MatchingCriterion MATCHING_DOMAIN = new MatchingDomainCriterion();
    public static final MatchingCriterion MATCHING_IP = new MatchingIPCriterion();
    public static final MatchingCriterion MATCHING_SECOND_LEVEL_DOMAIN = new MatchingSecondLevelDomainCriterion();

    public boolean isSatisfiedBy(final Domain mxRecord, final EmailHeader header) {
        return header.fromDomain.isValid() && satisfies(mxRecord, header);
    }

    protected abstract boolean satisfies(Domain mxRecord, EmailHeader header);

    private static class MatchingDomainCriterion extends MatchingCriterion {
        @Override
        protected boolean satisfies(final Domain mxRecord, final EmailHeader header) {
            return !(header.byIp.equals(header.fromIp) || header.fromIp.isLocalhost() || !header.byDomain.equals(mxRecord));
        }
    }

    private static class MatchingIPCriterion extends MatchingCriterion {
        @Override
        protected boolean satisfies(final Domain mxRecord, final EmailHeader header) {
            return header.byIp.equals(mxRecord.resolveIP());
        }
    }

    private static class MatchingSecondLevelDomainCriterion extends MatchingCriterion {
        @Override
        protected boolean satisfies(final Domain mxRecord, final EmailHeader header) {
            Domain secondLevelDomain = mxRecord.secondLevelDomain();
            return secondLevelDomain.equals(header.byDomain.secondLevelDomain());
        }
    }
}
