package org.agilefaqs.rft.internal;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.agilefaqs.rft.FakeNetwork;
import org.junit.Test;

public class DomainFactoryTest {
    private static final FakeNetwork FAKE_NETWORK = new FakeNetwork();

    @Test
    public void constructFromValidDomainNames() {
        Domain agileFAQs = parse("agilefaqs.com");
        assertTrue(agileFAQs.isValid());
    }

    @Test
    public void resolvesIPAddressToConstructDomain() {
        Domain google = parse("209.85.212.172");
        assertEquals("mail-vw0-f172.google.com", google.toString());
    }

    @Test
    public void invalidDomainNamesAreMappedToNullDomain() {
        Domain invalidDomain = parse("invalid.domain.name.123");
        assertFalse(invalidDomain.isValid());
    }

    @Test
    public void nonExistentDomainsAreMappedToNullIPAddress() {
        IPAddress ip = parse("this.domain.does.not.exist").resolveIP();
        assertFalse(ip.isValid());
    }

    @Test
    public void retrieveMappingDomainNameWhenNameIsIP() {
        Domain gogole = parse("209.85.212.172");
        assertEquals("mail-vw0-f172.google.com", gogole.toString());
    }

    @Test
    public void domainNameRetrivalFailuresResultInNullDomain() {
        Domain invalidDomain = parse("1.2.3.4");
        assertFalse(invalidDomain.isValid());
    }

    private Domain parse(final String domainName) {
        return DomainFactory.build(domainName, FAKE_NETWORK);
    }
}
