package org.agilefaqs.rft.internal;

import static org.junit.Assert.assertEquals;

import org.agilefaqs.rft.EmailHeader;
import org.agilefaqs.rft.FakeNetwork;
import org.junit.Test;

public class MatchingCriterionTest {
    private static final FakeNetwork FAKE_NETWORK = new FakeNetwork();
    private MatchingCriterion domainRule = MatchingCriterion.MATCHING_DOMAIN;
    private MatchingCriterion ipRule = MatchingCriterion.MATCHING_IP;
    private MatchingCriterion sldRule = MatchingCriterion.MATCHING_SECOND_LEVEL_DOMAIN;
    private static final Domain AGILE_INDIA = DomainFactory.build("mails.agileindia.org", FAKE_NETWORK);
    private boolean satisfies;
    private MatchingCriterion matchingCriterion;
    private EmailHeader header;

    @Test
    public void matchMxRecordDomainWithByDomain() {
        header = new EmailHeader("192.168.1.1", "agilefaqs.com", "192.168.2.1", "mails.agileindia.org", FAKE_NETWORK);
        assertThat(header).satisfies(domainRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void skipInvalidFromDomainsInHeader() {
        header = new EmailHeader("127.0.0.1", "invalid.domain.123", "192.168.2.1", "mails.agileindia.org", FAKE_NETWORK);
        assertThat(header).doesNotSatisfy(domainRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void skipLocalByDomains() {
        header = new EmailHeader("127.0.0.1", "agilefaqs.com", "192.168.2.1", "mails.agileindia.org", FAKE_NETWORK);
        assertThat(header).doesNotSatisfy(domainRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void skipHeadersWithMatchingByAndFromDomainIPs() {
        header = new EmailHeader("192.168.2.1", "agilefaqs.com", "192.168.2.1", "mails.agileindia.org", FAKE_NETWORK);
        assertThat(header).doesNotSatisfy(domainRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void matchIPAddressOfMxRecordWithByIP() {
        header = new EmailHeader("192.168.2.1", "agilefaqs.com", "72.14.203.121", "docs.agileindia.org", FAKE_NETWORK);
        assertThat(header).satisfies(ipRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void skipHeadersWhoseByIPDoesNotMatchMxRecordIP() {
        header = new EmailHeader("192.168.2.1", "agilefaqs.com", "1.2.3.4", "some.other.server.org", FAKE_NETWORK);
        assertThat(header).doesNotSatisfy(ipRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void matchSLDOfMxRecordWithSLDOfByDomain() {
        header = new EmailHeader("192.168.2.1", "agilefaqs.com", "72.14.203.121", "internal-mails.agileindia.org", FAKE_NETWORK);
        assertThat(header).satisfies(sldRule).forMxRecordOf(AGILE_INDIA);
    }

    @Test
    public void skipHeadersWhoseSLDByDomainDoesNotMatchSLDOfMxRecord() {
        header = new EmailHeader("192.168.2.1", "agilefaqs.com", "72.14.203.121", "agileindia.dreamhost.com", FAKE_NETWORK);
        assertThat(header).doesNotSatisfy(sldRule).forMxRecordOf(AGILE_INDIA);
    }

    private void forMxRecordOf(final Domain mxRecord) {
        assertEquals(satisfies, matchingCriterion.isSatisfiedBy(mxRecord, header));
    }

    private MatchingCriterionTest doesNotSatisfy(final MatchingCriterion m) {
        matchingCriterion = m;
        satisfies = false;
        return this;
    }

    private MatchingCriterionTest satisfies(final MatchingCriterion m) {
        matchingCriterion = m;
        satisfies = true;
        return this;
    }

    private MatchingCriterionTest assertThat(final EmailHeader header) {
        this.header = header;
        return this;
    }
}
